"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CustomNetworkTransform = void 0;
const Vector_1 = require("../../util/Vector");
const Networkable_1 = require("../Networkable");
class CustomNetworkTransform extends Networkable_1.Networkable {
    constructor(stage, netid, data) {
        super(stage, netid, data);
        this.classname = "CustomNetworkTransform";
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    Deserialize(reader, spawn = false) {
        const newSeqId = reader.uint16();
        if (!CustomNetworkTransform.seqIdGreaterThan(newSeqId, this.seqId)) {
            return;
        }
        this.seqId = newSeqId;
        this.position = CustomNetworkTransform.readVector2(reader);
        this.velocity = CustomNetworkTransform.readVector2(reader);
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    Serialize(writer, spawn = false) {
        writer.uint16(this.seqId);
        CustomNetworkTransform.writeVector2(writer, this.position);
        CustomNetworkTransform.writeVector2(writer, this.velocity);
    }
    static seqIdGreaterThan(newSid, oldSid) {
        if (typeof oldSid !== "number")
            return true;
        let margin = oldSid - CustomNetworkTransform.threshold;
        if (margin > 2 ** 16 - 1) {
            margin -= 2 ** 16;
        }
        if (oldSid < margin) {
            return newSid > oldSid && newSid <= margin;
        }
        return newSid > oldSid || newSid <= margin;
    }
    static writeVector2(writer, vector) {
        const x = Vector_1.Unlerp(vector.x) * 65535;
        const y = Vector_1.Unlerp(vector.y) * 65535;
        writer.uint16(x);
        writer.uint16(y);
    }
    static readVector2(reader) {
        const x = reader.uint16();
        const y = reader.uint16();
        return {
            x: Vector_1.Lerp(x / 65535),
            y: Vector_1.Lerp(y / 65535)
        };
    }
}
exports.CustomNetworkTransform = CustomNetworkTransform;
CustomNetworkTransform.classname = "CustomNetworkTransform";
CustomNetworkTransform.threshold = 2 ** 15 - 1;
